/*
 * Per-Hop Internet Traffic Measurement tools
 * Global definitions and data structures
 * 
 * Santosh Krishnan 3/01
 */

#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include<signal.h>
#include<errno.h>

#include<sys/time.h>
#include<sys/param.h>
#include<sys/socket.h>
#include<sys/file.h>
#include<sys/types.h>

#include<netinet/in_systm.h>
#include<netinet/in.h>
#include<netinet/ip.h>
#include<netinet/ip_icmp.h>
#include<netdb.h>

/* constants */
static char    *srcid = "Per-Hop Traffic Measurement Tool v1.0";
#define TRACEROUTE      "/usr/local/bin/traceroute"
#define ADDRSIZE        20
#define NAMESIZE        100
#define TRCMDSIZE       100
#define MAXHOPS         30
#define MINPROBES       10          /* Minimum allowed per batch     */
#define MINPROBEGAP     1           /* Atleast 1 sec between probes  */
#define RECVTIMEOUT     5           /* 5 sec   timeout for response  */
#define MAXLINKTIME     100000000

#define ICMPSIZE        ICMP_TSLEN  /* 20 bytes of ICMP header size  */
#define ICMPMICROSEC    1           /* For microseconds granularity  */
#define PACKSIZE        64          /* IP packet size for ICMP       */

/* compiler directives */
#define DEBUG_TIMING    1
/* #define DEBUG_ICMP      1 */
#define GUI_ENABLE      1
#ifdef GUI_ENABLE
#include<X11/Xlib.h>
#include<X11/Xutil.h>

#define WINWIDTH        700
#define WINHEIGHT       500
#define MAX_POINTS      50
#endif



/* 
 * Data structures 
 */

typedef struct struct_hop_entry {
  char       hop_name[NAMESIZE];    /* Name of host, if available    */
  char       hop_addr[ADDRSIZE];    /* IP address of host            */
} hop_entry;


/*
 * Data structure of a single probe 
 * Keeps the time-stamps for all the hops
 */
typedef struct struct_probe {
  u_long      T1;       /* Time-stamp of departure              */
  u_long      T2;       /* Time-stamp of arrival at remote host */
  u_long      T3;       /* Time-stamp of departure from remote  */
  u_long      T4;       /* Time-stamp of arrival of response    */
  int         rcvflag;  /* To indicate if timestamps are valid  */
} Probe;


/*
 * Data structure of average times and loss (single batch)
 * to be incorporated into results
 * All times are in milliseconds and loss is a fraction
 */
typedef struct struct_oqp {
  int        *o_time;         /* One for every hop */
  int        *q_time;  
  double     *p_loss;
  int        *d_time;         /* total symmetric delay */
  struct struct_oqp *next;    /* Next batch */
} oqp;


/*
 * Data structure for display
 * It contains assembled info from the oqp structures
 */
typedef struct struct_display {
  int        *l_time;         /* One for every hop */
  int        *o_time;   
  int        *q_time;
  int        *d_time;
  double     *p_loss;
} Display_Samples;


/*
 * Results file. Note that the l-time is
 * not averaged batch-wise
 */
typedef struct struct_result {
  int        num_batches;
  int        *l_time;         /* Propogation time for every link */
  oqp        *head_oqp;   
  oqp        *tail_oqp;
} Result;


/*
 * Global definitions
 */
/* main.c */
extern hop_entry              Hop_List[MAXHOPS];
extern int                    Total_Hops;

/* probe.c */
extern int                    Num_Probes;
extern int                    Probe_Gap;
extern void                   probe_kill (int);
extern pid_t                  Probe_Pid;
extern pid_t                  Meas_Pid;
extern int                    probe_count;

/* meas.c */
extern int                    Total_Batches;
extern int                    Total_Probes;
extern Probe                  **Batch_File;
extern int                    batch_count;
extern Result                 Result_File;
extern void                   meas_kill (int);
extern void                   update_batch (int);

/* icmp-proc.c*/
extern int                    icp_fd;
extern void                   icmp_send (int);

#ifdef GUI_ENABLE

typedef struct GUI_Display_Struct {
  int    num_points;             /* Number of points per hop */
  int    max_num_points;
  int    inter_point;            /* Inter-point interval     */
  int    max_point;              /* Maximum size of sample   */
  double scale;                  /* Scaling factor for points*/
  int    *d_time[MAX_POINTS];    /* Per Hop                  */
} GUI_Display;

extern int                    disp_fd;
#endif
