/*
 * All the GUI is handled in the functions here
 * Santosh Krishnan 3/01
 */

#include "def.h"

#ifdef GUI_ENABLE

int                   disp_fd;      /* Connection id for the display      */

static Display        *mydisplay;
static Window         rootwindow;
static int            rwidth, rheight;

static GC             mygc;  
static u_long         myforeground, mybackground;
static int            rwidth, twidth, pwidth; /* Region definitions       */
static int            rheight, theight, pheight;
static int            textx, texty;

GUI_Display           gui_Disp;     /* Display points for the GUI         */


/*
 * Initialize all the windows and start X
 * Called by: probe.c:: start_probes
 */
gui_init (int argc, char **argv)
{
  XSizeHints  roothint;
  int         myscreen;
  Font        font;
  XEvent      ev;
  
  /* Initialize display and screen */
  if (! (mydisplay = XOpenDisplay (""))) {
    printf ("Warning: cannot open display\n");
    exit (1);
  }
  disp_fd = ConnectionNumber (mydisplay);

  myscreen = DefaultScreen (mydisplay);
  mybackground = WhitePixel (mydisplay, myscreen);
  myforeground = BlackPixel (mydisplay, myscreen);

  /* Initialize root window */
  roothint.x = 200; roothint.y = 300;
  rwidth = roothint.width = WINWIDTH;
  rheight = roothint.height = WINHEIGHT;
  roothint.flags = PSize;

  rootwindow = XCreateSimpleWindow (mydisplay,
				    DefaultRootWindow (mydisplay),
				    roothint.x, roothint.y,
				    roothint.width, roothint.height,
				    5, myforeground, mybackground);
  XSetStandardProperties (mydisplay, rootwindow,
			  "tmtool", "tmtool", None, 
			  argv, argc, &roothint);

  /* Create graphics context */
  mygc = XCreateGC (mydisplay, rootwindow, 0, 0);
  XSetBackground (mydisplay, mygc, mybackground);
  XSetForeground (mydisplay, mygc, myforeground);
  XSetFunction (mydisplay, mygc, GXcopy);
  XSetLineAttributes (mydisplay, mygc, 1,
		      LineSolid, CapNotLast, JoinMiter);
  font = XLoadFont (mydisplay, "-*-times-medium-r-*-*-14-*"); 
  XSetFont (mydisplay, mygc, font);

  init_windows_regions ();
  /* Raise the window */
  XSelectInput (mydisplay, rootwindow, ExposureMask | StructureNotifyMask);
  XMapRaised (mydisplay, rootwindow);

  /*** start ***/
  process_gui ();
  return;
}



/*
 * Initialize regions in the window
 * for text strings and plots
 * Called by: gui_init
 */
init_windows_regions ()
{
  register int i,j;

  /* Define plot and string regions */
  theight = rheight/Total_Hops;
  twidth = 180;

  pheight = theight;
  pwidth = rwidth - twidth;

  textx = 10;
  texty = theight/2; /* Within a region */

  /* Initialize the GUI Display region */
  gui_Disp.num_points = 0;

  gui_Disp.inter_point = 20;             /* Inter-point width */
  gui_Disp.max_num_points = pwidth / gui_Disp.inter_point;
  if (gui_Disp.max_num_points > MAX_POINTS) {
    gui_Disp.max_num_points = MAX_POINTS;
    gui_Disp.inter_point = pwidth/ MAX_POINTS;
  }

  /* Scaling */
  gui_Disp.max_point = 1;
  gui_Disp.scale = (double) pheight / (double) gui_Disp.max_point;
  
  for (i=0; i<MAX_POINTS; i++) {
    gui_Disp.d_time[i] = (int *) malloc (sizeof (int) * Total_Hops);
    for (j=0; j<Total_Hops; j++)
      gui_Disp.d_time[i][j] = 0;
  }
  return;
}



/*
 * Process an event on the display
 * Called by: meas.c:: gui_meas_loop
 * Calls:gui_redraw
 */
process_gui ()
{
  XEvent ev;

  while (XEventsQueued (mydisplay, QueuedAfterFlush) > 0) {
    
    /* Check if this is an expose event for us */
    XNextEvent (mydisplay, &ev);
    
    switch (ev.type) {
    case Expose:
      if (ev.xexpose.count == 0)
	/* Just re-draw all the charts */
	gui_redraw ();
      break;
      
    case MappingNotify:
      /* This is needed! */
      XRefreshKeyboardMapping ((XMappingEvent *) &ev);
      break;
      
    case ConfigureNotify:
      /* Change height and width of regions */
      if ((ev.xconfigure.width != rwidth) ||
	  (ev.xconfigure.height != rheight)) {
	resize_regions (ev.xconfigure.width, ev.xconfigure.height);
	adjust_gui_display ();

	/* Redraw */
	gui_redraw ();
      }
      break;
    }
  }
  return;
}



/*
 * Width or Height of parent window changed
 * NOTE: There is a bug in the re-sizing of windows -- 03/27
 * Called by: gui_process
 */
resize_regions (int new_width, int new_height)
{
  rwidth = new_width;
  rheight = new_height;

  /* Define plot and string regions */
  theight = rheight/Total_Hops;
  twidth = 180;

  pheight = theight;
  pwidth = rwidth - twidth;

  textx = 10;
  texty = theight/2; /* Within a region */

  /* Change scaling factor */
  gui_Disp.scale = (double) pheight / (double) gui_Disp.max_point;

  gui_Disp.max_num_points = pwidth / gui_Disp.inter_point;
  if (gui_Disp.max_num_points > MAX_POINTS) {
    gui_Disp.max_num_points = MAX_POINTS;
    gui_Disp.inter_point = pwidth/ MAX_POINTS;
  }

  return;
}



/*
 * Redraw the entire plot
 * This is the main rendering function
 * Called by: gui.c:: process_gui
 *            meas.c::update_batch
 */
gui_redraw ()
{
  redraw_strings ();
  redraw_plots ();
  draw_regions ();
  return;
}



/*
 * Draw the lines separating the regions 
 * Called by gui_redraw
 */
draw_regions ()
{
  register int y=0;
  register int i;

  /* Vertical lines */
  XDrawLine (mydisplay, rootwindow, mygc, twidth, 0, twidth, rheight);

  /* Horizontal lines */
  for (i=0; i<Total_Hops; i++) {
    XDrawLine (mydisplay, rootwindow, mygc, 0, y, rwidth, y);
    y += theight;
  }
  return;
}



/*
 * Strings in the text regions
 * Called by gui_redraw
 */
redraw_strings ()
{
  register int y=texty;
  register int i;
  char     scale[20];

  /* Clear text area */
  XClearArea (mydisplay, rootwindow, 0, 0,
	      twidth, rheight, 0);

  /* The strings that name the nodes */
  for (i=0; i<Total_Hops; i++) {
    XDrawString (mydisplay, rootwindow, mygc, 20, y,
		 Hop_List[i].hop_name, strlen (Hop_List[i].hop_name));
    XDrawString (mydisplay, rootwindow, mygc, 20, y+15, 
		 Hop_List[i].hop_addr, ADDRSIZE);
    y += theight;
  }

  /* The string that prints the scale */
  sprintf (scale, "%7d (micro)", gui_Disp.max_point);
  XDrawString (mydisplay, rootwindow, mygc,
	       twidth - 100, 20,
	       scale, strlen (scale));
  return;
}



/*
 * Re-plot the plots in the plot regions
 * Called by: gui_redraw
 */
redraw_plots ()
{
  register int    i, j, x, y, num;
  register int    thickness, inter_point, point;
  register double scale;

  thickness = gui_Disp.inter_point / 2;

  /* Clear Plot region */
  XClearArea (mydisplay, rootwindow, twidth, 0, 
	      pwidth, rheight, 0);

  /* Process each plot region */
  scale = gui_Disp.scale;
  inter_point = gui_Disp.inter_point;
  y = pheight;
  num = gui_Disp.num_points;

  for (i=0; i<Total_Hops; i++) {

    /* For each point on each hop */
    x = twidth;
    for (j=0; j<num; j++) {

      point = gui_Disp.d_time[j][i];
      point = (int) (scale * (double) point);             /* Scale it */
      XFillRectangle (mydisplay, rootwindow, mygc,
		      x, y - point, thickness, point);
      x += inter_point;
    }
    y += pheight;
  }
  return;
}



/*
 * Adjust the points to be plotted
 * Change the num_points and scale, inter-point if necessary
 * Called by: meas.c:: update_meas
 *             gui.c:: process_gui
 */
adjust_gui_display ()
{
  register int  discarded=0;
  register oqp *tmp;
  register int  num, i, j, sample;

  /* Figure out the number of points */
  gui_Disp.num_points = Result_File.num_batches;
  if (gui_Disp.num_points > gui_Disp.max_num_points) {
    /* Truncate the results file */
    discarded = gui_Disp.num_points - gui_Disp.max_num_points;
    gui_Disp.num_points = gui_Disp.max_num_points;
  }

  /* Go ahead in the oqp list */
  for (i=0, tmp=Result_File.head_oqp; i<discarded; i++) 
    tmp = tmp->next;

  /* From here on, the filling of the gui_Display structure
     is exactly the same as filling the Disp structure in
     meas.c file one-by-one 
     */

  num = gui_Disp.num_points;
  for (i=0; i<num; i++) {
    /* For each hop */
    for (j=0; j<Total_Hops; j++) {
     
      /* For now, only d_time is plotted
	 and it is unadjusted 
	 */
      sample = gui_Disp.d_time[i][j] = tmp->d_time[j];

      /* Change scaling if necessary */
      if (sample > gui_Disp.max_point) {
	gui_Disp.max_point = sample;
	gui_Disp.scale = (double) pheight / (double) sample;
      }
    }
    tmp = tmp->next;
  }
  /* Now the gui_Disp structure is ready for plotting */
  return;
}



/*
 * Close all GUI resources
 * Called by: meas.c:: meas_kill
 */
gui_exit ()
{
  register int i;

  XFreeGC (mydisplay, mygc);
  XDestroyWindow (mydisplay, rootwindow);
  XCloseDisplay (mydisplay);
}
#endif
