/*
 * Send ICMP probes
 * Process replies and creates batch files
 * ready for measurement purposes
 *
 * Santosh Krishnan 03/01
 */

#include "def.h"

/* Probe sampling parameters */
int                    Num_Probes=MINPROBES;     /* Probes in a batch        */
int                    Probe_Gap=MINPROBEGAP;    /* Time (sec) between probes*/

/* Process control */
pid_t                  Probe_Pid;                /* Pid of prober            */
pid_t                  Meas_Pid;                 /* Pid of measurer          */
int                    probe_count=0;            /* only for debugging       */



/*
 * Initialize the probes for all the nodes
 * on the hop list
 * Called by main.c::main
 * Calls     probe.c::probe_loop, kill_probes 
 *           measure.c::init_batches, init_measures, meas_loop, kill_meas
 *           icmp-proc.c:: icmp_init
 */
start_probes (int argc, char **argv)
{
  pid_t pid;

  printf ("Starting Measurements..... \n");
  printf ("Number of probes in each batch to each hop: %d\n", Num_Probes);
  printf ("Inter-probe interval: %d (sec)\n", Probe_Gap);
  printf ("Number of hops: %d\n\n", Total_Hops);

  /* Compute the minimum number of batches to keep
     in order to allow for receive timeout
     */
  
  Total_Batches = 2 + (int) ((double) RECVTIMEOUT/
			     (double) (Num_Probes * Probe_Gap));
  Total_Probes = Total_Batches * Num_Probes; 
  icmp_init ();        /* Initialize all sockets */
  
  /* Fork a process. Handle sending probes in parent
     and handle batch results management and possibly GUI in the
     child */

  Probe_Pid = getpid ();

  if ((pid = fork ()) < 0)
    perror ("Fork error in start_probes");

  else if (pid > 0) {            /* Parent process, run probes */
    Meas_Pid = pid;
    signal (SIGCHLD, SIG_IGN);
    if (signal (SIGINT, probe_kill) == SIG_ERR)
      perror ("Error in setting signal handler in parent");
    probe_loop ();               /* Main probe sending routine */
  }

  else {                         /* Child process, run measurements */
    Meas_Pid = getpid ();
    if (signal (SIGINT, meas_kill) == SIG_ERR)
      perror ("Error in setting signal handler in child");

    init_batches ();
    init_measures ();
#ifdef GUI_ENABLE
    gui_init (argc, argv);
    gui_meas_loop ();
#else
    meas_loop ();                /* Main measurement routines */
#endif
  }
  return;
}


/*
 * Send probes
 * Called by probe.c:: start_probes
 * Calls: icmp-proc.c::icmp_send
 */
probe_loop ()
{
  sigset (SIGALRM, icmp_send);
  icmp_send (0);       /* Send the first batch of ICMP packets */
  for (;;) pause ();
  return;
}


/*
 * Handle graceful exit on key-board
 * interrupts. Wait for child to display results
 * Called by: probe.c
 */
void
probe_kill (int dummy)
{
  wait (0);  /* Wait for child */
  printf ("Ending Probes.... Total probe sets: %d\n", probe_count);
  printf ("Goodbye\n");
  exit (0);
}
